/***************************************************************************
 *   Copyright (C) 2005 by Andreas Silberstorff   *
 *   ml@andreas-silberstorff.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 *   In addition, as a special exception, the copyright holders give       *
 *   permission to link the code of this program with any edition of       *
 *   the Qt library by Trolltech AS, Norway (or with modified versions     *
 *   of Qt that use the same license as Qt), and distribute linked         *
 *   combinations including the two.  You must obey the GNU General        *
 *   Public License in all respects for all of the code used other than    *
 *   Qt.  If you modify this file, you may extend this exception to        *
 *   your version of the file, but you are not obligated to do so.  If     *
 *   you do not wish to do so, delete this exception statement from        *
 *   your version.                                                         *
 ***************************************************************************/
#include "kalva_player.h"
#include "settings.h"
#include "prefs.h"

#include <qstring.h>
#include <qlineedit.h>
#include <klocale.h>
#include <kprocess.h>
#include <kinputdialog.h>
#include <kapplication.h>

KalvaPlayer::KalvaPlayer(QWidget *parent, const char *name)
    : kalva_playergui(parent, name)
{
}

void KalvaPlayer::slotPlay()
{
    cmd   = playCmd();
    if ( Settings::show_command() == true  ) {
       bool ok;
       cmd = KInputDialog::getMultiLineText (
          i18n("Command to run"), 
          i18n("Command"), 
          cmd, 
          &ok,
          this,
          "cmdDlg"
       );
       if ( ok != true )  {
          return;
       }
   }
    KShellProcess *proc = new KShellProcess;
    *proc << cmd;
    proc->start( KProcess::DontCare );
}

void KalvaPlayer::stationChanged(const QString stat, const QString chan, const QString freq)
{
   m_station->setText(stat );
   m_channel->setText(chan );
   m_frq->setText(freq );
}

QString KalvaPlayer::playCmd()
{
    // BUILD the mplayer call
    bool    useFrq   = Settings::use_frq();
    QString audiodev = Settings::audiodev();
    QString tv_scale = Settings::tv_scale();
    x_scale    = tv_scale.section( ":", 0, 0 );
    y_scale    = tv_scale.section( ":", 1, 1 );
    y_scale    = y_scale.section( " ", 0, 0 );
    QString aumix    = Settings::aumix();

    cmd   = "";
    if ((audiodev != QString::null) and (audiodev.contains("[") <= 0)) {
       cmd.append(" " + aumix + " -" + audiodev + " unmute && ");
    }
    cmd.append(Settings::mplayer());
    cmd.append(" -bpp 32 ");

    cmd.append(" -fs tv:// -tv ");
    cmd.append(QString("driver=%1:norm=%2:device=%3:input=%4")
                  .arg(Settings::driver())
                  .arg(Settings::norm())
                  .arg(Settings::videodev())
                  .arg(Settings::input()));
    int brightness = Settings::brightness();
    if ( brightness > 0 ) {
       cmd.append(QString(":brightness=%1").arg(brightness));
    }
    int hue = Settings::hue();
    if ( hue  > 0 ) {
       cmd.append(QString(":hue=%1").arg(hue));
    }
    int saturation = Settings::saturation();
    if ( saturation > 0 )  {
       cmd.append( QString(":saturation=%1").arg( saturation ));
    }
    int contrast = Settings::contrast();
    if ( contrast > 0 ) {
       cmd.append(QString(":contrast=%1").arg(contrast));
    }
    //cmd.append(":outfmt=yuy2:quality=0");
    cmd.append(":quality=0");
    cmd.append(":width=" + x_scale + ":height=" + y_scale);
    //cmd.append(":fps=25");
    if (Settings::use_frq() == true )  {
       cmd.append(":freq=" + m_frq->text());
    }  else  {
       cmd.append(":channel=" + m_channel->text());
    }
    cmd.append(" -vo xv -v 3 ");

   /************************************************
    ************************************************
    ****  F i l t e r s
    ************************************************
    ************************************************/
    //cmd.append(" -vf pp=hb/vb/dr/al/lb");
    cmd.append(" -vf pp=fd");

   /************************************************
    * denoiser
    ************************************************/
    QString denoiser = Settings::denoiser();
    if (! denoiser.isEmpty()) {
       cmd.append( " -vf " + denoiser + "=" );
       cmd.append( QString("%1:%2:%3")
                           .arg( Settings::denoise3d_chroma() )
                           .arg( Settings::denoise3d_luma() )
                           .arg( Settings::denoise3d_time() )
                           );
    }
   /************************************************
    * crop
    ************************************************/
    QString crop = Settings::crop();
    if (! crop.isEmpty()) {
        cmd.append(" -vf rectangle=" + crop );
    }
   /************************************************
    * scale
    ************************************************/
    QString filterScale = Settings::filter_scale();
    if ((! (filterScale.isEmpty())) and 
        (filterScale != QString::null) and
        (filterScale.contains("\[") <= 0 ))  {
        filterScale    = filterScale.section( " ", 0, 0 );
        cmd.append(" -vf scale=" + filterScale);
    }
    if ((audiodev != QString::null) and (audiodev.contains("\[") <= 0)) {
       cmd.append(" && " + aumix + " -" + audiodev + " mute");
    }
    return cmd;
}

void KalvaPlayer::play ()
{
   slotPlay();
}

#include "kalva_player.moc"
