/***************************************************************************
 *   Copyright (C) 2005 by Andreas Silberstorff   *
 *   ml@andreas-silberstorff.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 *   In addition, as a special exception, the copyright holders give       *
 *   permission to link the code of this program with any edition of       *
 *   the Qt library by Trolltech AS, Norway (or with modified versions     *
 *   of Qt that use the same license as Qt), and distribute linked         *
 *   combinations including the two.  You must obey the GNU General        *
 *   Public License in all respects for all of the code used other than    *
 *   Qt.  If you modify this file, you may extend this exception to        *
 *   your version of the file, but you are not obligated to do so.  If     *
 *   you do not wish to do so, delete this exception statement from        *
 *   your version.                                                         *
 ***************************************************************************/

#include "kalva_recorder.h"
#include "settings.h"
#include "prefs.h"

#include <math.h>

#include <qcheckbox.h>
#include <qstring.h>
#include <qlineedit.h>
#include <qtabwidget.h>

#include <kdebug.h>
#include <klocale.h>
#include <kprocess.h>
#include <kmessagebox.h>
#include <ktimewidget.h>
#include <kdatepicker.h>
#include <kcombobox.h>
#include <kinputdialog.h>
#include <kapplication.h>

KalvaRecorder::KalvaRecorder(QWidget* parent, const char* name, WFlags fl)
    : kalva_recordergui(parent,name,fl)
{
   connect( m_film, SIGNAL( activated( const QString& )),
            m_film, SLOT( addToHistory( const QString& )));
   m_film->setMaxCount(100);
   m_film->setDuplicatesEnabled (false);
}

KalvaRecorder::~KalvaRecorder()
{}

/*$SPECIALIZATION$*/
void KalvaRecorder::setWeekends()
{
   m_monday->setChecked(false);
   m_tuesday->setChecked(false);
   m_wednesday->setChecked(false);
   m_thursday->setChecked(false);
   m_friday->setChecked(false);
   m_saturday->setChecked(false);
   m_sunday->setChecked(false);
}

void KalvaRecorder::setLaborDays()
{
   m_monday->setChecked(true);
   m_tuesday->setChecked(true);
   m_wednesday->setChecked(true);
   m_thursday->setChecked(true);
   m_friday->setChecked(true);
   m_saturday->setChecked(false);
   m_sunday->setChecked(false);
}

void KalvaRecorder::setDaily()
{
   m_monday->setChecked(true);
   m_tuesday->setChecked(true);
   m_wednesday->setChecked(true);
   m_thursday->setChecked(true);
   m_friday->setChecked(true);
   m_saturday->setChecked(true);
   m_sunday->setChecked(true);
}

void KalvaRecorder::stationChanged(const QString stat, const QString chan, const QString freq)
{
   m_station->setText(stat );
   m_channel->setText(chan );
   m_frq->setText(freq );
}

void KalvaRecorder::record()
{
   int recordMode = m_recordMode->currentPageIndex();
   switch (recordMode)
   {
      case 0:
         slotRecordNow();
         return;
      case 1:
         slotRecordMovie();
         return;
      case 2:
         slotRecordSerial();
         return;
   }
   cmd.append("***\nrecordMode = ");
   cmd.append(recordMode);
   KMessageBox::error ( this, cmd, i18n ("unsupported recordmode") );
}


void KalvaRecorder::slotRecordNow()
{
    cmd = recordCmd();

    if ( Settings::show_command() == true  ) {
       bool ok;
       cmd = KInputDialog::getMultiLineText (
          i18n("Command to run"), 
          i18n("Command"), 
          cmd, 
          &ok,
          this,
          "cmdDlg"
       );
       if ( ok != true )  {
          return;
       }
   }

   KShellProcess *proc = new KShellProcess;
   *proc << cmd;
   proc->start( KProcess::DontCare );
}

void KalvaRecorder::slotRecordMovie()
{
    cmd = recordCmd();
    if ( cmd.isEmpty() )  {
       return;
    }
    cmd.replace(QChar('"'), "\\\"");
    cmd.prepend("echo ");
    cmd.append(" | at " + m_startMovie->time().toString ("hh:mm") + " " +
                          m_calender->date().toString("dd.MM.yyyy") );

    if ( Settings::show_command() == true ) {
       bool ok;
       cmd = KInputDialog::getMultiLineText (
          i18n("Command to run"), 
          i18n("Command"), 
          cmd, 
          &ok,
          this,
          "cmdDlg"
       );
       if ( ok != true  )  {
          return;
       }
   }

    KShellProcess *proc = new KShellProcess;
    *proc << cmd;
    proc->start( KProcess::DontCare );
}

void KalvaRecorder::slotRecordSerial()
{
    cmd = recordCmd();
    if ( cmd.isEmpty() )  {
       return;
    }
    cmd.replace(QChar('"'), "\\\"");
    cmd.prepend("perl /usr/bin/tvapp.pl -film \"" + m_film->currentText() + "\"" +
                                         " -hour "          +
                                         m_startSerial->time().toString("h") + 
                                         " -minute "        +
                                         m_startSerial->time().toString("m") + 
                                         " -daysoftheweek " + setDOWs() +
                                         " -cmd \""
               );
    cmd.append("\" -cronaddcmd ");

    if ( Settings::show_command() == true ) {
       bool ok;
       cmd = KInputDialog::getMultiLineText (
          i18n("Command to run"), 
          i18n("Command"), 
          cmd, 
          &ok,
          this,
          "cmdDlg"
       );
       if ( ok != true  )  {
          return;
       }
   }

    KShellProcess *proc = new KShellProcess;
    *proc << cmd;
    proc->start( KProcess::DontCare );
}

QString KalvaRecorder::setDOWs()  {
   dow = "";
   dow = setDayInDOWs ( m_sunday->isChecked(),    dow, ((QString)"0"));
   dow = setDayInDOWs ( m_monday->isChecked(),    dow, ((QString)"1"));
   dow = setDayInDOWs ( m_tuesday->isChecked(),   dow, ((QString)"2"));
   dow = setDayInDOWs ( m_wednesday->isChecked(), dow, ((QString)"3"));
   dow = setDayInDOWs ( m_thursday->isChecked(),  dow, ((QString)"4"));
   dow = setDayInDOWs ( m_friday->isChecked(),    dow, ((QString)"5"));
   dow = setDayInDOWs ( m_saturday->isChecked(),  dow, ((QString)"6"));
   ///@TODO: translation if there is a sequence in the list (f.i."1,2,3,4,5" should become "1-5")
   return dow;
}

QString KalvaRecorder::setDayInDOWs ( bool isChecked, QString dow, QString day )
{
   if (isChecked == true )  {
      if (! (dow.isEmpty() ))   {
          dow.append(",");
      }
      dow.append(day);
   }
   return dow;
}

///@TODO: translation if there is a sequence in the list (f.i."1,2,3,4,5" should become "1-5")
void KalvaRecorder::setYourDOWs ( const QString & yourDOWs )
{
   QStringList dows = QStringList::split( ",", yourDOWs, false );
   QString dow;
   bool ok;
   unsetDOWs();
   for ( QStringList::Iterator it = dows.begin(); it != dows.end(); ++it ) {
       dow = *it;
       kdDebug() << "dow = "  << dow << endl;
       setYourDayInDOWs ( dow.toInt( &ok, 10 ) );
   }
}

void KalvaRecorder::unsetDOWs ()
{
   m_sunday->setChecked(false);
   m_monday->setChecked(false);
   m_tuesday->setChecked(false);
   m_wednesday->setChecked(false);
   m_thursday->setChecked(false);
   m_friday->setChecked(false);
   m_saturday->setChecked(false);
}

void KalvaRecorder::setYourDayInDOWs ( const int & yourDOW )
{
   switch (yourDOW) {
      case 0: { m_sunday->setChecked(true);
                break; }
      case 1: { m_monday->setChecked(true);
                break; }
      case 2: { m_tuesday->setChecked(true);
                break; }
      case 3: { m_wednesday->setChecked(true);
                break; }
      case 4: { m_thursday->setChecked(true);
                break; }
      case 5: { m_friday->setChecked(true);
                break; }
      case 6: { m_saturday->setChecked(true);
                break; }
   }
}


// BUILD the mencoder call
QString KalvaRecorder::recordCmd()
{
    bool    useFrq   = Settings::use_frq();

    tv_scale     = Settings::tv_scale();
    x_scale      = tv_scale.section( ":", 0, 0 );
    y_scale      = tv_scale.section( ":", 1, 1 );
    y_scale      = y_scale.section( " ", 0, 0 );

    QString audiodev = Settings::audiodev();
    QString aumix    = Settings::aumix();

    QString path     = Settings::storage_path();
    ///@TODO system ("mkdir -p $pfad") unless ( -d "$pfad" );

    QString film = QString("\"" + path + "/" + m_film->currentText() +
                   "_`" + Settings::date() + " +%a_%d.%b.%G_%H%M%S`_" +
                   m_station->text() +
                   ".avi\"");

    cmd = "";
    if ((audiodev != QString::null) and (audiodev.contains("[") <= 0)) {
        cmd.append(aumix + " -" + audiodev + " mute && " +
                   aumix + " -" + audiodev + " cap && ");
    }
    int nice_value  = Settings::nice_value();
    if (nice_value > 0)  {
       cmd.append(Settings::sudo());
       cmd.append(" ");
       cmd.append(Settings::nice());
       cmd.append(" -n " + nice_value );
       cmd.append(" ");
    }
    cmd.append(Settings::mencoder());
    // Hardwaresettings
    cmd.append(" tv:// ");
    cmd.append(QString("-tv driver=%1:norm=%2:device=%3:input=%4")
                  .arg(Settings::driver())
                  .arg(Settings::norm())
                  .arg(Settings::videodev())
                  .arg(Settings::input()));

    //cmd.append(":fps=25");

    // these are rather hardwareoptions as they are used for cards with ugly defaults
    int brightness = Settings::brightness();
    if ( brightness > 0 ) {
       cmd.append(QString(":brightness=%1").arg(brightness));
    }
    int hue = Settings::hue();
    if ( hue  > 0 ) {
       cmd.append(QString(":hue=%1").arg(hue));
    }
    int saturation = Settings::saturation();
    if ( saturation > 0 )  {
       cmd.append(":saturation=" + saturation);
    }
    int contrast = Settings::contrast();
    if ( contrast > 0 ) {
       cmd.append(QString(":contrast=%1").arg(contrast));
    }

    // qualityoptions
    //cmd.append(":outfmt=yuy2");
    cmd.append(":quality=0");
    cmd.append(":width="  + x_scale +
               ":height=" + y_scale);
    if (useFrq == true )  {
       cmd.append(":freq=" + m_frq->text() );
    }  else  {
       cmd.append(":channel=" + m_channel->text() );
    }

   /************************************************
    * lavc options
    ************************************************/
    cmd.append(" -ovc lavc -lavcopts vcodec=mpeg4");
    double bitrate  = setBitrate();
    cmd.append(QString(":vbitrate=%1").arg(bitrate));
    int vratetol = Settings::vratetol();
    if ( vratetol > 0 ) {
       cmd.append(QString(":vratetol=%1").arg(vratetol));
    }
    int mbd = Settings::mbd();
    if ( mbd > 0 ) {
       cmd.append(QString(":mbd=%1").arg(mbd));
    }
    int precmp = Settings::precmp();
    if ( precmp > 0 ) {
       cmd.append(QString(":precmp=%1").arg(precmp));
    }
    int cmp = Settings::cmp();
    if ( cmp > 0 ) {
       cmd.append(QString(":cmp=%1").arg(cmp));
    }
    int subcmp = Settings::subcmp();
    if ( subcmp > 0 ) {
       cmd.append(QString(":subcmp=%1").arg(subcmp));
    }
    QString aspect = Settings::aspect();
    if (! aspect.isEmpty()) {
       cmd.append(QString(":aspect=%1").arg(aspect));
    }
    if (Settings::grey() == true)  {
       cmd.append(":sw");
    }

   /************************************************
    * mp3 audio options
    ************************************************/
    //cmd.append(":keyint=25");
    cmd.append(" -oac mp3lame -lameopts ");
    //cmd.append("cbr:br=128:mode=0");
    cmd.append("abr:br=128:mode=0");

   /************************************************
    ************************************************
    ****  F i l t e r s
    ************************************************
    ************************************************/
    //cmd.append(" -vf pp=hb/vb/dr/al/lb");
    cmd.append(" -vf pp=fd");

   /************************************************
    * denoiser
    ************************************************/
    QString denoiser = Settings::denoiser();
    if (! denoiser.isEmpty()) {
       cmd.append( " -vf " + denoiser + "=" );
       cmd.append( QString("%1:%2:%3")
                           .arg( Settings::denoise3d_chroma() )
                           .arg( Settings::denoise3d_luma() )
                           .arg( Settings::denoise3d_time() )
                           );
    }
   /************************************************
    * crop
    ************************************************/
    QString crop = Settings::crop();
    if (! crop.isEmpty()) {
        cmd.append(" -vf crop=" + crop );
    }
   /************************************************
    * scale
    ************************************************/
    QString filterScale = Settings::filter_scale();
    if ((! (filterScale.isEmpty())) and 
        (filterScale != QString::null) and
        (filterScale.contains("\[") <= 0 ))  {
        filterScale    = filterScale.section( " ", 0, 0 );
        cmd.append(" -vf scale=" + filterScale);
    }

   /************************************************
    ************************************************
    **** other options
    ************************************************
    ************************************************/
    //QString duration = m_duration->time().toString();
    cmd.append(" -endpos " + m_duration->time().toString());

   /************************************************
    ************************************************
    **** optional use of aumix
    ************************************************
    ************************************************/
    cmd.append(" -o " + film);
    if ((audiodev != QString::null) and (audiodev.contains("\[") <= 0)) {
        cmd.append(" && " + aumix + " -" + audiodev + " unmute && " +
                            aumix + " -" + audiodev + " nocap ");
    }
    return cmd;
}

double KalvaRecorder::setBitrate()
{
  int hours   = m_duration->time().hour();
  int minutes = m_duration->time().minute();
  int seconds = m_duration->time().second();
  double bitrate = 2000;
  minutes     = minutes + (hours * 60);
  if ((minutes > 0) and (seconds > 0))  {
     minutes -= 1;
  }
  // computation of the vbitrate: up to 128 minutes we use 2000 kByte/s,
  // from 129 minutes on we compute a lower bitrate to keep the
  // resulting filesize <= 2 GB 
  if ( minutes > 128 )
     bitrate = floor(((1995 - 0.9375 * minutes) * 8192/(60 * minutes)));
  return bitrate;
}

QString KalvaRecorder::recordNow(
                    const QString & yourTitel,
                    const QString & yourLength
                    )
{
   m_recordMode->setCurrentPage(0);
   m_film->setCurrentText( yourTitel );
   m_duration->setTime( QTime::fromString ( yourLength ) );
   slotRecordNow();
   return i18n("recording immediatly");
}

QString KalvaRecorder::recordMovie(
                    const QString & yourTitel,
                    const QString & yourLength,
                    const QString & yourStartTime,
                    const QString & yourStartDate
                    )
{
   ///@TODO find out how we can put the recorder itself on top
   m_recordMode->setCurrentPage(1);
   m_film->setCurrentText( yourTitel );
   m_duration->setTime( QTime::fromString ( yourLength ) );
   m_startMovie->setTime( QTime::fromString ( yourStartTime ) );
   ///@TODO make this functional!
   m_calender->setDate( QDate::fromString ( yourStartDate, Qt::ISODate ) );
   slotRecordMovie();
   return i18n("recording Movie");
   //return QString("ERROR:%1").arg(i18n("So sorry, not implemented yet!"));
}

QString KalvaRecorder::recordSerial(
                    const QString & yourTitel,
                    const QString & yourLength,
                    const QString & yourStartTime,
                    const QString & yourDOWs
                    )
{
   ///@TODO find out how we can put the recorder itself on top
   m_recordMode->setCurrentPage(2);
   m_film->setCurrentText( yourTitel );
   m_duration->setTime( QTime::fromString ( yourLength ) );
   m_startSerial->setTime( QTime::fromString ( yourStartTime ) );
   setYourDOWs( yourDOWs );
   slotRecordSerial();
   return i18n("recording serial");
}

#include "kalva_recorder.moc"
